@@ -801,9 +801,6 @@ neon_rule_loadpX (OrcCompiler *compiler, void *user, OrcInstruction *insn)
     } else if (size == 4) {
       orc_neon_emit_loadpl (compiler, dest->alloc, insn->src_args[0]);
     } else if (size == 8) {
-      if (src->size == 8) {
-        ORC_COMPILER_ERROR(compiler,"64-bit parameters not implemented");
-      }
       orc_neon_emit_loadpq (compiler, dest->alloc, insn->src_args[0]);
     } else {
       ORC_PROGRAM_ERROR(compiler,"unimplemented");
@@ -1018,6 +1015,7 @@ neon_rule_storeX (OrcCompiler *compiler, void *user, OrcInstruction *insn)
 }
 
 
+#if 0
 static int
 orc_neon_get_const_shift (unsigned int value)
 {
@@ -1029,6 +1027,7 @@ orc_neon_get_const_shift (unsigned int value)
   }
   return shift;
 }
+#endif
 
 void
 orc_neon_emit_loadib (OrcCompiler *compiler, int reg, int value)
@@ -1057,30 +1056,29 @@ void
 orc_neon_emit_loadiw (OrcCompiler *compiler, int reg, int value)
 {
   orc_uint32 code;
-  int shift;
-  int neg = FALSE;
 
   if (value == 0) {
     orc_neon_emit_binary_quad (compiler, "veor", 0xf3000110, reg, reg, reg);
     return;
   }
 
-  if (value < 0) {
-    neg = TRUE;
-    value = ~value;
-  }
-  shift = orc_neon_get_const_shift (value);
-  if ((value & (0xff<<shift)) == value) {
-    value >>= shift;
-    if (neg) {
-      ORC_ASM_CODE(compiler,"  vmvn.i16 %s, #%d\n",
-          orc_neon_reg_name_quad (reg), value);
-      code = 0xf2800830;
-    } else {
-      ORC_ASM_CODE(compiler,"  vmov.i16 %s, #%d\n",
-          orc_neon_reg_name_quad (reg), value);
-      code = 0xf2800810;
-    }
+  ORC_ASM_CODE(compiler,"  vmov.i16 %s, #0x%04x\n",
+      orc_neon_reg_name_quad (reg), value & 0xff);
+  code = 0xf2800810;
+  code |= (reg&0xf) << 12;
+  code |= ((reg>>4)&0x1) << 22;
+  code |= (value&0xf) << 0;
+  code |= (value&0x70) << 12;
+  code |= (value&0x80) << 17;
+  code |= 0x40;
+  orc_arm_emit (compiler, code);
+
+  value >>= 8;
+  if (value) {
+    ORC_ASM_CODE(compiler,"  vorr.i16 %s, #0x%04x\n",
+        orc_neon_reg_name_quad (reg), (value & 0xff) << 8);
+    code = 0xf2800b10;
+
     code |= (reg&0xf) << 12;
     code |= ((reg>>4)&0x1) << 22;
     code |= (value&0xf) << 0;
@@ -1088,54 +1086,38 @@ orc_neon_emit_loadiw (OrcCompiler *compiler, int reg, int value)
     code |= (value&0x80) << 17;
     code |= 0x40;
     orc_arm_emit (compiler, code);
-
-    if (shift > 0) {
-      ORC_ASM_CODE(compiler,"  vshl.i16 %s, %s, #%d\n",
-          orc_neon_reg_name_quad (reg), orc_neon_reg_name_quad (reg), shift);
-      code = 0xf2900510;
-      code |= (reg&0xf) << 12;
-      code |= ((reg>>4)&0x1) << 22;
-      code |= (reg&0xf) << 0;
-      code |= ((reg>>4)&0x1) << 5;
-      code |= (shift&0xf) << 16;
-      code |= 0x40;
-      orc_arm_emit (compiler, code);
-    }
-
-    return;
   }
 
-  ORC_COMPILER_ERROR(compiler, "unimplemented load of constant %d", value);
 }
 
 void
 orc_neon_emit_loadil (OrcCompiler *compiler, int reg, int value)
 {
   orc_uint32 code;
-  int shift;
-  int neg = FALSE;
 
   if (value == 0) {
     orc_neon_emit_binary_quad (compiler, "veor", 0xf3000110, reg, reg, reg);
     return;
   }
 
-  if (value < 0) {
-    neg = TRUE;
-    value = ~value;
-  }
-  shift = orc_neon_get_const_shift (value);
-  if ((value & (0xff<<shift)) == value) {
-    value >>= shift;
-    if (neg) {
-      ORC_ASM_CODE(compiler,"  vmvn.i32 %s, #%d\n",
-          orc_neon_reg_name_quad (reg), value);
-      code = 0xf2800030;
-    } else {
-      ORC_ASM_CODE(compiler,"  vmov.i32 %s, #%d\n",
-          orc_neon_reg_name_quad (reg), value);
-      code = 0xf2800010;
-    }
+  ORC_ASM_CODE(compiler,"  vmov.i32 %s, #0x%08x\n",
+      orc_neon_reg_name_quad (reg), value & 0xff);
+  code = 0xf2800010;
+
+  code |= (reg&0xf) << 12;
+  code |= ((reg>>4)&0x1) << 22;
+  code |= (value&0xf) << 0;
+  code |= (value&0x70) << 12;
+  code |= (value&0x80) << 17;
+  code |= 0x40;
+  orc_arm_emit (compiler, code);
+
+  value >>= 8;
+  if (value & 0xff) {
+    ORC_ASM_CODE(compiler,"  vorr.i32 %s, #0x%08x\n",
+        orc_neon_reg_name_quad (reg), (value & 0xff) << 8);
+    code = 0xf2800310;
+
     code |= (reg&0xf) << 12;
     code |= ((reg>>4)&0x1) << 22;
     code |= (value&0xf) << 0;
@@ -1143,24 +1125,35 @@ orc_neon_emit_loadil (OrcCompiler *compiler, int reg, int value)
     code |= (value&0x80) << 17;
     code |= 0x40;
     orc_arm_emit (compiler, code);
-
-    if (shift > 0) {
-      ORC_ASM_CODE(compiler,"  vshl.i32 %s, %s, #%d\n",
-          orc_neon_reg_name_quad (reg), orc_neon_reg_name_quad (reg), shift);
-      code = 0xf2a00510;
-      code |= (reg&0xf) << 12;
-      code |= ((reg>>4)&0x1) << 22;
-      code |= (reg&0xf) << 0;
-      code |= ((reg>>4)&0x1) << 5;
-      code |= (shift&0x1f) << 16;
-      code |= 0x40;
-      orc_arm_emit (compiler, code);
-    }
-
-    return;
   }
+  value >>= 8;
+  if (value & 0xff) {
+    ORC_ASM_CODE(compiler,"  vorr.i32 %s, #0x%08x\n",
+        orc_neon_reg_name_quad (reg), (value & 0xff) << 16);
+    code = 0xf2800510;
 
-  ORC_COMPILER_ERROR(compiler, "unimplemented load of constant %d", value);
+    code |= (reg&0xf) << 12;
+    code |= ((reg>>4)&0x1) << 22;
+    code |= (value&0xf) << 0;
+    code |= (value&0x70) << 12;
+    code |= (value&0x80) << 17;
+    code |= 0x40;
+    orc_arm_emit (compiler, code);
+  }
+  value >>= 8;
+  if (value & 0xff) {
+    ORC_ASM_CODE(compiler,"  vorr.i32 %s, #0x%08x\n",
+        orc_neon_reg_name_quad (reg), (value & 0xff) << 24);
+    code = 0xf2800710;
+
+    code |= (reg&0xf) << 12;
+    code |= ((reg>>4)&0x1) << 22;
+    code |= (value&0xf) << 0;
+    code |= (value&0x70) << 12;
+    code |= (value&0x80) << 17;
+    code |= 0x40;
+    orc_arm_emit (compiler, code);
+  }
 }
 
 void
@@ -1289,7 +1282,24 @@ orc_neon_emit_loadpq (OrcCompiler *compiler, int dest, int param)
       update ? "!" : "");
   code = 0xf4a0000d;
   code |= 2<<10;
-  code |= (0&7)<<5;
+  code |= (0)<<7;
+  code |= (compiler->gp_tmpreg&0xf) << 16;
+  code |= (dest&0xf) << 12;
+  code |= ((dest>>4)&0x1) << 22;
+  code |= (!update) << 1;
+  orc_arm_emit (compiler, code);
+
+  ORC_ASM_CODE(compiler,"  vld1.32 %s[0], [%s]%s\n",
+      orc_neon_reg_name (dest+1),
+      orc_arm_reg_name (compiler->gp_tmpreg),
+      update ? "!" : "");
+  code = 0xf4a0000d;
+  code |= 2<<10;
+  code |= (0)<<7;
+  code |= (compiler->gp_tmpreg&0xf) << 16;
+  code |= ((dest+1)&0xf) << 12;
+  code |= (((dest+1)>>4)&0x1) << 22;
+  code |= (!update) << 1;
   orc_arm_emit (compiler, code);
 
   orc_arm_emit_add_imm (compiler, compiler->gp_tmpreg,
@@ -1302,7 +1312,24 @@ orc_neon_emit_loadpq (OrcCompiler *compiler, int dest, int param)
       update ? "!" : "");
   code = 0xf4a0000d;
   code |= 2<<10;
-  code |= (1&7)<<5;
+  code |= (1)<<7;
+  code |= (compiler->gp_tmpreg&0xf) << 16;
+  code |= (dest&0xf) << 12;
+  code |= ((dest>>4)&0x1) << 22;
+  code |= (!update) << 1;
+  orc_arm_emit (compiler, code);
+
+  ORC_ASM_CODE(compiler,"  vld1.32 %s[1], [%s]%s\n",
+      orc_neon_reg_name (dest+1),
+      orc_arm_reg_name (compiler->gp_tmpreg),
+      update ? "!" : "");
+  code = 0xf4a0000d;
+  code |= 2<<10;
+  code |= (1)<<7;
+  code |= (compiler->gp_tmpreg&0xf) << 16;
+  code |= ((dest+1)&0xf) << 12;
+  code |= (((dest+1)>>4)&0x1) << 22;
+  code |= (!update) << 1;
   orc_arm_emit (compiler, code);
 }
 
@@ -2062,7 +2089,8 @@ orc_neon_rule_mergebw (OrcCompiler *p, void *user, OrcInstruction *insn)
           p->vars[insn->src_args[0]].alloc);
     }
 
-    if (p->vars[insn->src_args[1]].last_use != p->insn_index) {
+    if (p->vars[insn->src_args[1]].last_use != p->insn_index ||
+        p->vars[insn->src_args[1]].alloc == p->vars[insn->dest_args[0]].alloc) {
       orc_neon_emit_mov (p, p->tmpreg, p->vars[insn->src_args[1]].alloc);
       orc_neon_emit_unary (p, "vzip.8", 0xf3b20180,
           p->vars[insn->dest_args[0]].alloc,
@@ -2094,7 +2122,8 @@ orc_neon_rule_mergewl (OrcCompiler *p, void *user, OrcInstruction *insn)
           p->vars[insn->src_args[0]].alloc);
     }
 
-    if (p->vars[insn->src_args[1]].last_use != p->insn_index) {
+    if (p->vars[insn->src_args[1]].last_use != p->insn_index ||
+        p->vars[insn->src_args[1]].alloc == p->vars[insn->dest_args[0]].alloc) {
       orc_neon_emit_mov (p, p->tmpreg, p->vars[insn->src_args[1]].alloc);
       orc_neon_emit_unary (p, "vzip.16", 0xf3b60180,
           p->vars[insn->dest_args[0]].alloc,
@@ -2110,7 +2139,8 @@ orc_neon_rule_mergewl (OrcCompiler *p, void *user, OrcInstruction *insn)
           p->vars[insn->src_args[0]].alloc);
     }
 
-    if (p->vars[insn->src_args[1]].last_use != p->insn_index) {
+    if (p->vars[insn->src_args[1]].last_use != p->insn_index ||
+        p->vars[insn->src_args[1]].alloc == p->vars[insn->dest_args[0]].alloc) {
       orc_neon_emit_mov_quad (p, p->tmpreg, p->vars[insn->src_args[1]].alloc);
       orc_neon_emit_unary_quad (p, "vzip.16", 0xf3b60180,
           p->vars[insn->dest_args[0]].alloc,
@@ -2132,7 +2162,8 @@ orc_neon_rule_mergelq (OrcCompiler *p, void *user, OrcInstruction *insn)
           p->vars[insn->src_args[0]].alloc);
     }
 
-    if (p->vars[insn->src_args[1]].last_use != p->insn_index) {
+    if (p->vars[insn->src_args[1]].last_use != p->insn_index ||
+        p->vars[insn->src_args[1]].alloc == p->vars[insn->dest_args[0]].alloc) {
       orc_neon_emit_mov (p, p->tmpreg, p->vars[insn->src_args[1]].alloc);
       orc_neon_emit_unary (p, "vtrn.32", 0xf3ba0080,
           p->vars[insn->dest_args[0]].alloc,
@@ -2148,7 +2179,8 @@ orc_neon_rule_mergelq (OrcCompiler *p, void *user, OrcInstruction *insn)
           p->vars[insn->src_args[0]].alloc);
     }
 
-    if (p->vars[insn->src_args[1]].last_use != p->insn_index) {
+    if (p->vars[insn->src_args[1]].last_use != p->insn_index ||
+        p->vars[insn->src_args[1]].alloc == p->vars[insn->dest_args[0]].alloc) {
       orc_neon_emit_mov_quad (p, p->tmpreg, p->vars[insn->src_args[1]].alloc);
       orc_neon_emit_unary_quad (p, "vzip.32", 0xf3ba0180,
           p->vars[insn->dest_args[0]].alloc,
